#############
# $Id: split_phix.pm 13527 2011-07-04 11:30:01Z ajb $
# Created By: gq1
# Last Maintained By: $Author: ajb $
# Created On: 2010-11-24
# Last Changed On: $Date: 2011-07-04 12:30:01 +0100 (Mon, 04 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.1/lib/npg_pipeline/archive/file/split_phix.pm $

package npg_pipeline::archive::file::split_phix;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13527 $ =~ /(\d+)/mxs; $r; };
use File::Spec;
use File::Basename;

extends q{npg_pipeline::archive::file::generation};

with qw{
  npg_pipeline::roles::business::pre_exec_strings
};

# leaving these two here - should be noted that this is found to be the most optimal
# way of running, so may need changing
Readonly::Scalar our $DEFAULT_NUMBER_OF_PROCESSORS => 4;
Readonly::Scalar our $DEFAULT_RESOURCES => q{-R 'select[mem>8000] rusage[mem=8000] span[hosts=1]' -M8000000 -n} . $DEFAULT_NUMBER_OF_PROCESSORS;
Readonly::Scalar our $REF_NAME          => q{phix};

sub split_run_lanes {
  my ($self, $arg_refs) = @_;

  my @lanes;
  my $run = $self->run();
  if ( $self->is_paired_read() ) {
    $arg_refs->{paired}++;
  }
  if ( $self->is_indexed() ) {
    $arg_refs->{multiplexed}++;
  }

  @lanes = $self->positions($arg_refs);

  my @wanted_lanes;

  foreach my $lane ( @lanes ) {
    if ( $self->_is_lane_spiked_phix($lane) eq q{true} ) {
      push @wanted_lanes, $lane;
    }
  }

  # get out if no lanes require splitting
  if ( ! scalar @wanted_lanes ) {
    $self->log('No spiked phix lane for this run, no job to split phix');
    return ();
  }

  $arg_refs->{array_string} = $self->create_array_string( @wanted_lanes );

  my $lane = $self->lsb_jobindex();

  my $job_id = $self->submit_bsub_command( $self->_generate_bsub_command( $lane, $arg_refs ) );

  return ( $job_id );

}

### private methods

###############
# responsible for generating the bsub command to be executed
sub _generate_bsub_command {
  my ($self, $lane, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();

  my $archive_dir = $self->archive_path();

  my $location_of_logs = $self->make_log_dir( $self->archive_path() );

  my ($fastq1, $fastq2, $fastqt);
  if ($arg_refs->{paired}) {
    $fastq1 = $self->fastq_file_constructs();
    $fastq1 =~ s/substitute_this/substitute_this_1/xms;
    $fastq1 =~ s/substitute_this/$lane/xms;
    $fastq2 = $self->fastq_file_constructs();
    $fastq2 =~ s/substitute_this/substitute_this_2/xms;
    $fastq2 =~ s/substitute_this/$lane/xms;
    if ($arg_refs->{multiplexed}) {
      $fastqt = $self->fastq_file_constructs();
      $fastqt =~ s/substitute_this/substitute_this_t/xms;
      $fastqt =~ s/substitute_this/$lane/xms;
    }
  } else {
    if ($arg_refs->{multiplexed}) {
      $fastq1 = $self->fastq_file_constructs();
      $fastq1 =~ s/substitute_this/substitute_this_1/xms;
      $fastq1 =~ s/substitute_this/$lane/xms;
      $fastqt = $self->fastq_file_constructs();
      $fastqt =~ s/substitute_this/substitute_this_t/xms;
      $fastqt =~ s/substitute_this/$lane/xms;
    } else {
      $fastq1 = $self->fastq_file_constructs();
      $fastq1 =~ s/substitute_this/$lane/xms;
    }
  }

  my $job_name = q{split_phix_} . $self->id_run() . q{_} . $timestamp;
  my $outfile = $location_of_logs . q{/} . $job_name . q{.%I.%J.out};
  $job_name .= $arg_refs->{array_string};

  my $bsub_command = q{bsub -q } . $self->lsf_queue() . qq{ $required_job_completion -J $job_name };
  $bsub_command .=  ( $self->fs_resource_string( {
    resource_string => $DEFAULT_RESOURCES,
    counter_slots_per_job => $DEFAULT_NUMBER_OF_PROCESSORS,
  } ) ) . q{ };
  $bsub_command .=  q{-o } . $outfile . q{ };
  $bsub_command .=  $self->ref_adapter_pre_exec_string() . q{ };
  $bsub_command .=  q{'} . $self->internal_script_bin() . q{/} . $self->external_script_names_conf()->{split_program_file} . q{ };

  $bsub_command .= qq{-fastq1 $archive_dir/$fastq1 };
  if ($arg_refs->{paired}) {
    $bsub_command .= qq{-fastq2 $archive_dir/$fastq2 };
  }
  if ($arg_refs->{multiplexed}) {
    $bsub_command .= qq{-fastqt $archive_dir/$fastqt };
  }
  $bsub_command .= qq{ -no_pileup -ref_name $REF_NAME};
  my $ref_bwa = $self->_get_reference({aligner=>q{bwa}, species => q{PhiX}, });
  my $ref_fasta = $self->_get_reference({aligner=>q{fasta}, species => q{PhiX},});

  $bsub_command .= q{ -ref_bwa }.$ref_bwa;
  $bsub_command .= q{ -ref_fa }.$ref_fasta;

  $bsub_command .= q{ -fork_alignment -id_run } . $self->id_run() . qq{ -position $lane -output_dir $archive_dir};

  $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastq1});
  $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastq1}.q{check});
  $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastq1}.q{.md5});

  if ($arg_refs->{paired}) {
    $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastq2});
    $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastq2}.q{check});
    $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastq2}.q{.md5});
  }

  if ($arg_refs->{multiplexed}) {
     $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastqt});
     $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastqt}.q{check});
     $bsub_command .= q{ && } . $self->_rename_file_commands(qq{$archive_dir/$fastqt}.q{.md5});
  }
  $bsub_command .= q{'}; #'

  if ($self->verbose()) {
    $self->log( $bsub_command) ;
  }
  return $bsub_command;
}

sub _rename_file_commands {
   my ($self, $file) = @_;

   my @suffix_list = qw(.fastq .fastqcheck .fastq.md5);
   my ($name, $path, $suffix) = fileparse($file, @suffix_list);

   my $original_filename = File::Spec->catfile($path, $name.q{_orig}.$suffix);
   my $nonphix_filename     = File::Spec->catfile($path, $name.q{_non}.$REF_NAME.$suffix);

   return qq{mv $file $original_filename && mv $nonphix_filename $file};
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::split_phix

=head1 VERSION

$LastChangedRevision: 13527 $

=head1 SYNOPSIS

  my $oSplitNC = npg_pipeline::archive::file::split_phix->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module for assessing which lanes require spiked phix to be removed and then bsubs jobs for these

=head1 SUBROUTINES/METHODS

=head2 split_run_lanes - method to call which determines which run lanes require splitting and then submits the jobs to LSF, returning an array of job_ids

  my @job_ids = $oSplitNC->split_run_lanes({
    required_job_completion => $sRequiredJobsToComplete,
    timestamp => $sTimeStamp,
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
