#############
# $Id: daemons.pm 13527 2011-07-04 11:30:01Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-09-15
# Last Changed On: $Date: 2011-07-04 12:30:01 +0100 (Mon, 04 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.1/lib/npg_pipeline/daemons.pm $

package npg_pipeline::daemons;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13527 $ =~ /(\d+)/mxs; $r; };
use npg::api::run_status_dict;

extends q{npg_pipeline::base};
with q{npg_pipeline::roles::business::source_files};

sub BUILD {
  my ($self) = @_;

  my $test_log_dir = $ENV{TEST_LOG} || q{};
  $self->set_log_file_path( $test_log_dir . $self->filesystem_locations_conf()->{central_log_file_dir} );

  my @temp = split /::/xms, (ref $self);
  $self->set_log_file_name( $temp[0] . q{_} . $temp[-1] . q{.log} );

  return 1;
}

##############
# Private methods

# switch $id_run to be the first end if necessary
sub _use_lowest_id_run {
  my ($self, $run) = @_;
  my $id_run = $run->id_run();
  my $id_run_pair = $run->id_run_pair();
  if ($id_run_pair < $id_run) {
    $id_run = $id_run_pair;
  }
  return $id_run;
}

################
# for caching submitted runs

has q{_seen}      => (isa => q{HashRef}, is => q{ro}, lazy_build => 1);

sub _build__seen {
  my ($self) = @_;
  return {};
}


has q{_api_util} => (isa => q{Object},  is => q{ro}, predicate => q{_has_api_util}, writer => q{_set_api_util});

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::daemons

=head1 VERSION

$LastChangedRevision: 13527 $

=head1 SYNOPSIS

  my $runner = npg_pipeline::daemons::<sub_class>->new();
  $runner->run();

=head1 DESCRIPTION

Base module for modules which daemons run

=head1 SUBROUTINES/METHODS

=head2 BUILD

Setups the logging on creation of the derived class object

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
