#############
# $Id: archival_runner.pm 13527 2011-07-04 11:30:01Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-09-14
# Last Changed On: $Date: 2011-07-04 12:30:01 +0100 (Mon, 04 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.1/lib/npg_pipeline/daemons/archival_runner.pm $

package npg_pipeline::daemons::archival_runner;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13527 $ =~ /(\d+)/mxs; $r; };
use npg::api::run_status_dict;

extends qw{npg_pipeline::daemons};

Readonly::Scalar our $POST_QC_REVIEW_SCRIPT       => q{npg_pipeline_post_qc_review};
Readonly::Scalar our $ARCHIVAL_PENDING            => q{archival pending};

sub run {
  my ($self) = @_;

  #################
  # obtain runs with a status of archival pending
  my $runs_to_upload = npg::api::run_status_dict->new({
    description => $ARCHIVAL_PENDING,
    ($self->_has_api_util() ? (q{util} => $self->_api_util()) : ()),
  })->runs();

  #################
  # go through each run, running the post_qc_review script on each
  # run on first instance of seeing an end of the run, and skipping the other end

  my $seen = $self->_seen();

  my $default_queue = $ENV{LSB_DEFAULTQUEUE} || q{(generic default)};

  $self->log(q{Default Queue is } . $default_queue);

  foreach my $run (@{$runs_to_upload}) {

    my $id_run = $run->id_run();
    # skip if you have already seen this
    if ($seen->{$id_run}) {
      $self->log(qq{Already seen $id_run});
      next;
    }

    my @run_ids = [$id_run];
    # check the pair, and assess which is the first end, as we only want to run on this
    if ($run->is_paired_run()) {
      my $lowest_id_run = $self->_use_lowest_id_run($run);
      $id_run = $lowest_id_run || $id_run;
      push @run_ids, $id_run;
    }

    eval {
      # create and launch the command
      my $cmd = $self->_generate_command($id_run);
      $self->log(qq{Running $cmd});
      my $output = `$cmd`;
      if ($CHILD_ERROR) {
        $self->log(qq{Output:\n$output});
        croak q{Problem running command};
      }
      foreach my $run_id (@run_ids) {
        $seen->{$run_id}++;
      }
    } or do {
      carp $EVAL_ERROR;
    };
  }

  return 1;
}

##############
# Private methods

# generate the command to run
sub _generate_command {
  my ($self, $id_run) = @_;

  return $self->all_source_files_as_a_source_string() . $self->internal_script_bin() . q{/} . $POST_QC_REVIEW_SCRIPT . q{ --verbose --id_run=} . $id_run;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::daemons::archival_runner

=head1 VERSION

$LastChangedRevision: 13527 $

=head1 SYNOPSIS

  my $runner = npg_pipeline::archival_runner->new();
  $runner->run();

=head1 DESCRIPTION

This module interrogates the npg database for runs with a status of archival pending, and then runs the npg_pipeline_post_qc_review script on each of them

=head1 SUBROUTINES/METHODS

=head2 run - the only method and the only one you need. It does everything.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_pipeline::daemons

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
