#############
# $Id: post_gerald.pm 13527 2011-07-04 11:30:01Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-06-18
# Last Changed On: $Date: 2011-07-04 12:30:01 +0100 (Mon, 04 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.1/lib/npg_pipeline/pluggable/post_gerald.pm $

package npg_pipeline::pluggable::post_gerald;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13527 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::pluggable};

Readonly::Scalar our $JOB_ID_10 => 10;
Readonly::Scalar our $JOB_ID_20 => 20;
Readonly::Scalar our $JOB_ID_30 => 30;
Readonly::Scalar our $JOB_ID_40 => 40;
Readonly::Scalar our $JOB_ID_45 => 45;
Readonly::Scalar our $JOB_ID_50 => 50;
Readonly::Scalar our $JOB_ID_60 => 60;

Readonly::Array our @FUNCTION_ORDER => (qw{
    create_archival_files
    spread_after_srf_creation
    default_align
    record_align_stats
    finish
  });

sub main {
  my ($self, @args) = @_;

  foreach my $function (@FUNCTION_ORDER) {
    $self->say(q{****** Processing }.$function.q{ ******});
    my @job_ids = $self->$function(@args);
    unshift @args, $self->lsf_job_complete_requirements(@job_ids);
  }

  return 1;
}

#################
# First in this pipeline fires off job to create archival files
#
sub create_archival_files {
  my ($self, @args) = @_;

  my $required_job_completion = shift @args;

  ##### fire off scripts to launch archival creation

  $self->say(qq{$required_job_completion - creating archival files});

  my $job_id = $JOB_ID_10;

  return ($job_id);
}

################
# After archival files have been created, a spread out of jobs, some of which end a pipeline chain
#
sub spread_after_srf_creation {
  my ($self, @args) = @_;

  my @job_ids;

  my $required_job_completion = $args[0];

  $self->say(q{spreading out});

  push @job_ids, $self->adapter_count(@args);
  push @job_ids, $self->sample_for_contamination(@args);
  push @job_ids, $self->quality_control(@args);
  my $remove_contamination_job_id = $self->remove_contamination(@args);
  if ($remove_contamination_job_id) {
    push @job_ids, $remove_contamination_job_id;
  }

  return @job_ids;
}

################
#  Count adapters
#
sub adapter_count {
  my ($self, @args) = @_;

  my $required_job_completion = shift @args;

  ##### fire off scripts for adapter_counting

  my $job_id = $JOB_ID_20;

  $self->say(qq{$required_job_completion - Adapter Counting});

  return $job_id;
}

###############
# Take sample to determine possible contamination
#
sub sample_for_contamination {
  my ($self, @args) = @_;

  my $required_job_completion = shift @args;

  ##### fire off scripts for contamination sample

  my $job_id = $JOB_ID_30;

  $self->say(qq{$required_job_completion - sampling for contamination});

  return $job_id;
}

##############
# Run quality controm scripts and fire off reports to SeqScape
#
sub quality_control {
  my ($self, @args) = @_;

  my $required_job_completion = shift @args;

  ###### fire off scripts for quality control
  my $job_id = $JOB_ID_40;

  $self->say(qq{$required_job_completion - performing quality control});

  return $job_id;
}

############
# Check to see if the project is expecting any contamination to be removed, and do so
#
sub remove_contamination {
  my ($self, @args) = @_;

  ####### check requirement to remove contamination
  $self->say(q{testing to see if contamination removal required});

  my $remove_contamination = 0;

  if ($remove_contamination) {

    my $required_job_completion = shift @args;

    $self->say(qq{$required_job_completion - removing contamination});
    ######### fire off scripts to remove contamination
    my $job_id = $JOB_ID_45;

    return $job_id;
  }

  return;
}

############
# Run default alignments, if reference seqs are available
#
sub default_align {
  my ($self, @args) = @_;

  my $required_job_completion = shift @args;

  ####### fire off scripts for default align
  $self->say(qq{$required_job_completion - doing default alignments});


  my $job_id = $JOB_ID_50;

  return $job_id;
}

############
# Record alignment stats
#
sub record_align_stats {
  my ($self, @args) = @_;

  my $required_job_completion = shift @args;

  ###### fire of scripts to record alignment stats
  $self->say(qq{$required_job_completion - recording alignment stats});

  my $job_id = $JOB_ID_60;

  return $job_id;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

  npg_pipeline::pluggable::post_gerald

=head1 VERSION

$LastChangedRevision: 13527 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 main - method called which calls all the functions in the order given, working out job dependencies for the next in line to pass through

=head2 adapter_count
=head2 create_archival_files
=head2 default_align
=head2 quality_control
=head2 record_align_stats
=head2 remove_contamination
=head2 sample_for_contamination
=head2 spread_after_srf_creation

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Readonly

=item base

=item npg_pipeline::pluggable

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
