#############
# $Id: other_params.pm 13527 2011-07-04 11:30:01Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-11-02
# Last Changed On: $Date: 2011-07-04 12:30:01 +0100 (Mon, 04 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.1/lib/npg_pipeline/roles/generic/other_params.pm $

package npg_pipeline::roles::generic::other_params;
use Moose::Role;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13527 $ =~ /(\d+)/mxs; $r; };

=head1 NAME

npg_pipeline::roles::generic::other_params

=head1 VERSION

$LastChangedRevision: 13527 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::generic::other_params};

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 other_params

use to pass through as a string other parameters that you would like (perhaps needed for a later script, etc)

  my $sOtherParams = $class->other_params();

=head2 other_params_hash

If the string you passed to other_params is of the form key1=val1:key2=val2:..., this will return a hash of those key value pairs

  my $hOtherParamsHash = $class->other_params_hash();

If you provide a method _process_other_params which will parse your other_params string into a hashref, then you can still use this method to store and retrieve

=cut

has q{other_params} => (isa => q{Str}, is => q{ro},
                        documentation => q{other parameters set as a string},);

has q{other_params_hash} => (isa => q{HashRef}, is => q{ro}, lazy_build => 1,
                             documentation => q{Do not populate on construction},);

sub _build_other_params_hash {
  my ($self) = @_;
  return $self->_process_other_params();
}

################
# generic method for processing other_params either to or from a string in format
# key1=val1:key2=val2:...
#
sub _process_other_params {
  my ($self) = @_;

  my @keyval_pairs = split /:/xms, $self->other_params();
  my $other_params = {};
  foreach my $keyval_pair (@keyval_pairs) {
    my @temp = split /=/xms, $keyval_pair;
    $other_params->{$temp[0]} = $temp[1];
  }
  return $other_params;
}

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
