# $Id: 10-pluggable_harold.t 13527 2011-07-04 11:30:01Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 20;
use Test::Deep;
use Test::Exception;
use lib qw{t};
use t::util;
use File::Slurp;

my $util = t::util->new();
$util->catch_email();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{OWNING_GROUP} = q{staff};

use_ok('npg_pipeline::pluggable::harold');

my $analysis_runfolder_path = $util->analysis_runfolder_path();
$ENV{TEST_USERAGENT} = $analysis_runfolder_path . q{/useragent};

{
  my $harold;
  my $functions_in_order = [qw(
      create_archive_directory
      create_fastq
      create_fastqcheck
      create_gcfreq
      create_md5
      finish
    )];
  lives_ok {
    $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 1234,
      function_order => $functions_in_order,
      run_folder => q{123456_IL2_1234},
      analysis_path => q{Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto},
      runfolder_path => $analysis_runfolder_path,
    });
  } q{no croak on creation};
  isa_ok($harold, q{npg_pipeline::pluggable::harold}, q{$harold});
  is($harold->function_order(), $functions_in_order, q{$harold->function_order() set on creation});

}

$util->set_staging_analysis_area({with_latest_summary => 1});
write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

{
  my $harold;
  lives_ok {
    $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      domain => q{test},
      conf_path => q{data/config_files},
    });
  } q{no croak on new creation};
  my @function_order;
  lives_ok {
    @function_order = @{$harold->_function_order()};
  } q{no croak setting function order};
  is_deeply(\@function_order, [ qw{finish} ], q{function order to be used would be base default});

  $harold->_clear_function_order;

  $harold->precreated_function_order(q{spam});
  lives_ok {
    @function_order = @{$harold->_function_order()};
  } q{no croak setting function order};
  is_deeply(\@function_order, [ qw{finish} ], q{function order to be used would be base default when a value for precreated_function_order is not an accepted one});

  $harold->_clear_function_order;

  $harold->precreated_function_order(q{archive_run});
  is($harold->precreated_function_order(), q{archive_run}, q{$harold->precreated_function_order() set});
  lives_ok {
    @function_order = @{$harold->_function_order()};
  } q{no croak setting function order};
  is_deeply(\@function_order, $harold->function_order_conf()->{post_qc_review}, q{function order to be used would be post_qc_review});

  my $parallelise_hash;
  lives_ok {
    $parallelise_hash = $harold->parallelise();
  } q{croak obtaining parallelise_hash};
}

{
  my $harold = npg_pipeline::pluggable::harold->new({
      schema => q{harold},
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      domain => q{test},
      conf_path => q{data/config_files},});

  cmp_deeply($harold->_autoqc_options_from_args(), {}, 'autoqc options: empty hash if no arguments');
  my $expected = {some_key => 'some_value',};
  cmp_deeply($harold->_autoqc_options_from_args($expected), $expected, 'autoqc options: options do not change if no args array');
  cmp_deeply($harold->_autoqc_options_from_args($expected, ()), $expected, 'autoqc options: options do not change if args array empty');
  cmp_deeply($harold->_autoqc_options_from_args($expected, qw/dodo/), $expected, 'autoqc options: options do not change if wrong array el type');
  cmp_deeply($harold->_autoqc_options_from_args($expected, ($expected)), $expected, 'autoqc options: options do not change if wrong hash');
  cmp_deeply($harold->_autoqc_options_from_args(
    {some_key => 'some_value',}, ({autoqc_data => q[some_data],})), $expected, 'autoqc options: options do not change if wrong hash');

  $expected->{my_key} = q[my_value];
  cmp_deeply($harold->_autoqc_options_from_args(
    {some_key => 'some_value',}, ({autoqc_data => {my_key => 'my_value',},})), $expected, 'autoqc options: options do not change if wrong hash');
}

1;