# $Id: 20-archive_file-correct_qseq.t 13527 2011-07-04 11:30:01Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 12;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::correct_qseq');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

{
  my $c_qseq;
  lives_ok { $c_qseq = npg_pipeline::archive::file::correct_qseq->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{created object ok};
  isa_ok($c_qseq, q{npg_pipeline::archive::file::correct_qseq}, q{$c_qseq});
  $util->create_analysis();
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  $ENV{TEST_API_UTIL} = $util->apiutil();

  lives_ok { $c_qseq->_create_qseq_lane_dirs(q{staff}); } q{directory created ok};

  throws_ok { $c_qseq->_generate_bsub_command($arg_refs) } qr{You[ ]have[ ]not[ ]provided[ ]an[ ]index[ ]length[ ]and[ ]there[ ]is[ ]no[ ]preset[ ]tag_file.}, q{_generate_bsub_command throws as no index_length or tag_file provided};

  $c_qseq->tag_file(q{/Path/To/Test/File});
  is($c_qseq->get_tag_file(), q{/Path/To/Test/File}, q{tag_file_retrieved when set and no index provided});
  $c_qseq->_clear_tag_file();
  throws_ok { $c_qseq->get_tag_file({index_length => 100}); } qr{There[ ]is[ ]no[ ]default[ ]tag_file[ ]for[ ]index[ ]length[ ]100}, q{throws as index length does not have a default};
  
  my @jids;
  lives_ok { @jids = $c_qseq->correct($arg_refs); } q{no croak running correct method};
  is(scalar@jids, 0, q{as this is not an index run, returns no job ids});

  $c_qseq->_test_use_bases('Y37,I6n,y37');
  $c_qseq->tag_file(q{t/barcodes/five_bp.tags});
  my $bsub_command = $util->drop_temp_part_from_paths( $c_qseq->_generate_bsub_command($arg_refs) );
  my $expected_cmd = q{bsub -q test -w'done(123) && done(321)' -J tag_qseqs_1234__20090709-123456 -R 'rusage[nfs_12=1]' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/tag_qseqs/tag_qseqs_1234__20090709-123456.out 't/bin/software/solexa/bin/index_decoder -t t/barcodes/five_bp.tags -u Y37,I6n,y37 -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/tag_qseqs /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_1_1_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_1_2_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_2_1_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_3_1_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_4_1_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_5_1_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_6_1_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_7_1_qseq_custom.txt /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_8_1_qseq_custom.txt > /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/tag_qseqs/index_decoder_.out'};
  is( $bsub_command, $expected_cmd, q{generated bsub command is correct} );

  lives_ok { @jids = $c_qseq->correct($arg_refs); } q{no croak running correct method};
  is(scalar@jids, 8, q{set to be an index run, 8 job ids in array});

}
1;