# $Id: 30-run_folder-move.t 13527 2011-07-04 11:30:01Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 16;
use Test::Exception;
use lib qw{t};
use Digest::MD5;
use t::useragent;
use t::util;
use File::Slurp;

my $util = t::util->new({});
$util->catch_email();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
my $tmp_dir = $util->temp_directory();
#$ENV{TEST_FS_RESOURCE} = q{nfs_12};

$ENV{TEST_USERAGENT} = $tmp_dir . q{/useragent};
write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

use_ok('npg_pipeline::run::folder::move');

{
  my $rfm;
  my $run_folder = q{123456_IL2_1234};
  my $run_name   = q{IL2_1234};
  my $runfolder_path = $util->analysis_runfolder_path();
  lives_ok {
    $rfm = npg_pipeline::run::folder::move->new( {
      run_folder => $run_folder,
      conf_path => $conf_path,
      domain => q{test},
      runfolder_path => $runfolder_path,
    } );
  } q{no croak as run folder provided};

  my $in_dir;
  lives_ok {
    $in_dir = $rfm->_get_instrument_dir( $run_name );
  } q{no croak obtaining instrument_dir};
  is($rfm->_get_instrument_dir( q{IL2_1235} ), $in_dir, q{new cache ok});

  my @dirs = split m{/}xms, $in_dir;
  shift @dirs;
  `rm -rf /$dirs[0]/$dirs[1]/$dirs[2]`;
  `mkdir -p /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]/$dirs[4]/outgoing`;
  `mkdir -p $runfolder_path`;

  lives_ok {
    $rfm->_move_individual_folder( $run_name );
  } q{no croak moving folder};

  throws_ok {
    $rfm->_move_individual_folder( $run_name );
  } qr{Failed[ ]to[ ]find[ ]$runfolder_path[ ]to[ ]move}, q{croaked as no folder in analysis to be found to move};

  `mkdir -p $runfolder_path`;

  throws_ok {
    $rfm->_move_individual_folder( $run_name );
  } qr{/tmp/.*?/nfs/sf45/IL2/outgoing/123456_IL2_1234[ ]already[ ]exists}, q{croaked as folder already exists in outgoing};

  `rm -rf /$dirs[0]/$dirs[1]`;

}

{
  my $rfm;
  my $run_folder = q{123456_IL2_1234};
  my $folder     = q{outgoing};
  lives_ok {
    $rfm = npg_pipeline::run::folder::move->new({
      run_folder => $run_folder,
      folder     => $folder,
      timestamp  => q{20090709-123456},
      verbose    => 1,
      conf_path => $conf_path,
      domain => q{test},
      runfolder_path => $util->analysis_runfolder_path(),
    });
  } q{folder attribute populated on new ok};
  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my $bsub_command = $util->drop_temp_part_from_paths( $rfm->_generate_bsub_command($arg_refs) );
  my $expected_cmd = q{bsub -w'done(123) && done(321)' -J move_run_folder_123456_IL2_1234_20090709-123456 -q small -R 'rusage[nfs_12=1]' -o /nfs/sf45/IL2/outgoing/log/123456_IL2_1234_move_run_folder_123456_IL2_1234_20090709-123456.out 't/bin/software/solexa/bin/move_run_folder --conf_path data/config_files --domain test --folder outgoing --run_folder 123456_IL2_1234 --runfolder_path /nfs/sf45/IL2/analysis/123456_IL2_1234 --verbose '};

  is($bsub_command, $expected_cmd, q{bsub command ok when moving to outgoing dir to be used});

  `mkdir -p $tmp_dir/nfs/sf45/IL2/analysis`;

  $rfm->_set_folder(q{analysis});
  $bsub_command = $util->drop_temp_part_from_paths( $rfm->_generate_bsub_command($arg_refs) );
  $expected_cmd = q{bsub -w'done(123) && done(321)' -J move_run_folder_123456_IL2_1234_20090709-123456 -q small -R 'rusage[nfs_12=1]' -o /nfs/sf45/IL2/analysis/log/123456_IL2_1234_move_run_folder_123456_IL2_1234_20090709-123456.out 't/bin/software/solexa/bin/move_run_folder --conf_path data/config_files --domain test --folder analysis --fs_resource nfs_12 --internal_script_bin t/bin/software/solexa/bin --name IL2_1234 --run_folder 123456_IL2_1234 --runfolder_path /nfs/sf45/IL2/analysis/123456_IL2_1234 --small_lsf_queue small --verbose '};
  is($bsub_command, $expected_cmd, q{bsub command ok when moving to analysis dir to be used});

  lives_ok { $rfm->submit_move_run_folder($arg_refs) } q{job id returned ok from $rfm->submit_move_run_folder()};
  `rm -rf $tmp_dir/nfs/sf45`;
}

{
  my $rfm;
  my $run_folder = q{123456_IL2_2341};
  my $run_name   = q{IL2_2341};
  my $rf_path = $util->analysis_runfolder_path();
  $rf_path =~ s/1234\z/2341/xms;

  lives_ok {
    $rfm = npg_pipeline::run::folder::move->new({
      run_folder => $run_folder,
      conf_path => $conf_path,
      domain => q{test},
      runfolder_path => $rf_path,
    });
  } q{no croak as run folder provided};

  my $in_dir;
  lives_ok {
    $in_dir = $rfm->_get_instrument_dir($run_name);
  } q{no croak obtaining instrument_dir};

  my @dirs = split m{/}xms, $in_dir;
  shift @dirs;
  `rm -rf /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]`;
  `mkdir -p /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]/$dirs[4]/analysis/$run_folder; mkdir /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]/$dirs[4]/analysis/123456_IL2_3412; mkdir /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]/$dirs[4]/outgoing; `;
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

  lives_ok {
    $rfm->move_runfolder();
  } q{no croak moving folders};
}

{
  my $run_folder = q{123456_IL2_2341};
  my $run_name   = q{IL2_2341};
  my $rfm = npg_pipeline::run::folder::move->new({
      run_folder => $run_folder,
      conf_path => $conf_path,
      domain => q{test},
      runfolder_path => $util->analysis_runfolder_path(),
    });

  my $in_dir = $rfm->_get_instrument_dir($run_name);
  my @dirs = split m{/}xms, $in_dir;
  shift @dirs;
  `rm -rf /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]`;
  `mkdir -p /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]/$dirs[4]/analysis/$run_folder/Data/Intensities/Bustard_RTA/GERALD_RTA; mkdir /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]/$dirs[4]/analysis/123456_IL2_3412; mkdir /$dirs[0]/$dirs[1]/$dirs[2]/$dirs[3]/voutgoing; `;
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );
  lives_ok { qx[bin/move_run_folder --conf_path $conf_path --domain live --verbose --run_folder=$run_folder --folder=outgoing]; } q{no croak running create_summary_link};
  ok(!$CHILD_ERROR, qq{Error code ok : $CHILD_ERROR});
}
1;