#########
# Author:        David K Jackson <david.jackson@sanger.ac.uk>
# Maintainer:    $Author: jo3 $
# Created:       2010-10-20
# Last Modified: $Date: 2009-09-30 09:54:57 +0100 (Wed, 30 Sep 2009) $
# Id:            $Id: 40-create_compressed_image_heirarchy.t 6600 2009-09-30 08:54:57Z jo3 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/data_handling/trunk/t/40-create_compressed_image_heirarchy.t $
#

use strict;
use warnings;
use English q(-no_match_vars);
use Carp;
use Test::More;
eval {require Test::Cmd; 1} or plan skip_all => 'Test::Cmd missing';
plan tests => 20;
use File::Basename qw(dirname);
use File::Spec::Functions qw(catfile rel2abs catdir);
use Perl6::Slurp;
use File::Temp qw(tempdir);
use Cwd;
our $VERSION = do { my ($r) = q$LastChangedRevision: 6600 $ =~ /(\d+)/smx; $r; };

my $origdir = getcwd;
note qq{origdir: $origdir};
my $basedir = rel2abs( dirname( dirname(__FILE__) ) );
note qq{basedir: $basedir};
my $qseq2fastq_path = catfile( $basedir, qw(bin qseq2fastq.pl) );
note "qseq2fastq_path: $qseq2fastq_path";

my $testdir = tempdir( CLEANUP => 1 );
note qq{testdir: $testdir};
my $testrun = Test::Cmd->new( prog => $qseq2fastq_path, verbose => 0, workdir => q(qseq2fastq_test) );
chdir $testdir or croak "Cannot change to temporary directory $testdir";
ok( $testrun, 'creating Test::Cmd object' );

# check to see if md5sum and fastqcheck are available to run. if not, then we shall skip/modify some tests
qx{echo 'foo' | md5sum};
my $has_no_md5sum = $CHILD_ERROR;

my $cmd = q{echo '@} . q{foo' | fastqcheck};
qx{$cmd};
my $has_no_fastqcheck = $CHILD_ERROR;


{
  my $id_run = 5371;
  my $datasrcdir = catdir( $basedir, qw(t data qseq2fastq), $id_run );
  #diag "datasrcdir: $datasrcdir";

  $testrun->run( args => '--filter ' . catfile( $datasrcdir, 's_1_1_0019_qseq_snippet.txt' ) );
  cmp_ok( $?, '==', 0, 'successfully run qseq2fastq with filter' );
  like( $testrun->stderr, qr/read 20 reads, wrote 12/, 'stderr reports records output and filtered' );
  is( $testrun->stdout, slurp( catfile( $datasrcdir, $id_run . '_1_1.fastq' ) ), 'stdout gives fastq' );

  my $args_string = q{--filter };
  if ( $has_no_md5sum ) {
    diag 'unable to test md5 option, as no md5sum available';
  } else {
    $args_string .= qq{--md5 ${id_run}_1_2.fastq.md5 };
  }
  if ( $has_no_fastqcheck ) {
    diag 'unable to test fastqcheck option, as no fastqcheck available';
  } else {
    $args_string .= qq{--fastqcheck ${id_run}_1_2.fastqcheck };
  }
  $args_string .= catfile( $datasrcdir, 's_1_2_0019_qseq_snippet.txt' ) ;

  note $args_string;
  $testrun->run( args => $args_string );
  cmp_ok( $?, '==', 0, 'successfully run qseq2fastq with filter, md5 and fastqcheck' );
  like( $testrun->stderr, qr/read 20 reads, wrote 12/, 'stderr reports records output and filtered' );
  is( $testrun->stdout, slurp( catfile($datasrcdir, $id_run . '_1_2.fastq' ) ), 'stdout gives fastq' );

  SKIP: {
    skip( 'no md5sum program available', 1 ) if ( $has_no_md5sum );
    like( slurp( "${id_run}_1_2.fastq.md5"), qr/\bc17ddfbd898bd30eb2b518e26f67ebea\b/, 'correct md5' );
  }
  SKIP: {
    skip( 'no fastqcheck program available', 1 ) if ( $has_no_fastqcheck );
    is( slurp( "${id_run}_1_2.fastqcheck"), slurp( catfile( $datasrcdir, "${id_run}_1_2.fastqcheck" ) ), 'correct fastqcheck' );
  }
}

{
  my $id_run = 5352;
  my $datasrcdir = catdir( $basedir, qw(t data qseq2fastq), $id_run );
  #diag "datasrcdir: $datasrcdir";

  $testrun->run( args => '--filter --index ' . catfile( $datasrcdir, 's_1_1_0[01]19_qseq_snippet.txt' ) );
  cmp_ok( $?, '==', 0, 'successfully run qseq2fastq with filter, index, and multiple input qseq' );
  like( $testrun->stderr, qr/read 40 reads, wrote 32/, 'stderr reports records output and filtered' );
  is( $testrun->stdout, slurp( catfile( $datasrcdir,$id_run.'_1_1.fastq' ) ), 'stdout gives fastq' );

  $testrun->run( args => "--filter --read t --index " . catfile( $datasrcdir, 's_1_2_0[01]19_qseq_snippet.txt' ) );
  cmp_ok( $?, '==', 0, 'successfully run qseq2fastq with filter, read, index, and multiple input qseq' );
  like( $testrun->stderr, qr/read 40 reads, wrote 32/, 'stderr reports records output and filtered' );
  is( $testrun->stdout, slurp(catfile( $datasrcdir, $id_run . '_1_t.fastq' ) ), 'stdout gives fastq' );

  my $args_string = q{--filter };
  if ( $has_no_md5sum ) {
    diag 'unable to test md5 option, as no md5sum available';
  } else {
    $args_string .= qq{--md5 ${id_run}_1_2.fastq.md5 };
  }
  if ( $has_no_fastqcheck ) {
    diag 'unable to test fastqcheck option, as no fastqcheck available';
  } else {
    $args_string .= qq{--fastqcheck ${id_run}_1_2.fastqcheck };
  }
  $args_string .= q{--read 2 --index } . catfile( $datasrcdir, 's_1_3_0[01]19_qseq_snippet.txt' );

  note $args_string;
  $testrun->run( args => $args_string );
  cmp_ok( $?, '==', 0, 'successfully run qseq2fastq with filter, md5, fastqcheck, read, index, and multiple input qseq' );
  like( $testrun->stderr, qr/read 40 reads, wrote 32/, 'stderr reports records output and filtered' );
  is( $testrun->stdout, slurp( catfile( $datasrcdir, $id_run . '_1_2.fastq' ) ), 'stdout gives fastq' );
  SKIP: {
    skip( 'no md5sum program available', 1 ) if ( $has_no_md5sum );
    like(slurp( "${id_run}_1_2.fastq.md5"), qr/\ba67b6d941ec2feec5fe55e8b95c94f58\b/, 'correct md5' );
  }
  SKIP: {
    skip( 'no fastqcheck program available', 1 ) if ( $has_no_fastqcheck );
    is(slurp( "${id_run}_1_2.fastqcheck"), slurp( catfile( $datasrcdir,"${id_run}_1_2.fastqcheck" ) ), 'correct fastqcheck' );
  }
}

chdir $origdir or croak "could not return to orginal directory $origdir";
1;
